
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Date;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.InetAddress;
import sun.net.NetworkServer;
import sun.net.www.MessageHeader;

public class HttpServer extends NetworkServer {

    MessageHeader mh;
    URL defaultContext;
    String docRoot = "/htdocs";
    final static String indexfile = "index.html";
    final static String version = "1.0";
    final static String mime_text_plain = "text/plain";
    final static String mime_text_html = "text/html";
    final static String mime_image_gif = "image/gif";
    final static String mime_image_jpg = "image/jpg";
    final static String mime_app_os = "application/octet-stream";

    final int buffer_size = 8192;

    HttpServer() throws MalformedURLException {
	defaultContext = new URL("http", "localhost", "/");
    }

    protected void getRequest(URL u) {
	String fname = u.getFile();
	try {
	    fname = docRoot + fname + (fname.endsWith("/") ? indexfile : "");
	    InputStream is = new FileInputStream(fname);
	    int size = is.available();

	    String ct;
	    if (fname.endsWith(".html"))
		ct = mime_text_html;
	    else if (fname.endsWith(".class"))
		ct = mime_app_os;
	    else if (fname.endsWith(".gif"))
		ct = mime_image_gif;
	    else if (fname.endsWith(".jpg"))
		ct = mime_image_jpg;
	    else
		ct = mime_text_plain;

	    System.out.println("GET " + fname + " " + size + " " + ct);

	    mimeHeader(ct, size);

	    byte buf[] = new byte[buffer_size];
	    int n;
	    while ((n = is.read(buf)) >= 0) {
		clientOutput.write(buf, 0, n);
	    }
	    is.close();
	    clientOutput.close();
	} catch(FileNotFoundException e) {
	    error(404, "Not Found", fname);
	} catch(IOException e) {
	    error(403, "Forbidden", fname);
	} catch(Exception e) {
	    error(500, "Unknown", fname);
	}
    }

    protected void mimeHeader(String ct, int size) {
	mimeHeader(200, "OK", ct, size);
    }

    protected void mimeHeader(int code, String msg, String ct, int size) {
	Date d = new Date();
	clientOutput.print("HTTP/1.0 " + code + " " + msg + "\r\n" +
			   "Date: " + d.toGMTString() + "\r\n" +
			   "Server: Java/" + version + "\r\n" +
			   "Content-type: " + ct + "\r\n" +
			   (size>0 ? "Content-length: " + size + "\r\n" : "") +
			   "\r\n");
    }

    protected void htmlHeader(String title) {
	clientOutput.print("<html>\n<head>\n<title>" + title +
			   "</title>\n</head>");
    }

    protected void error(int code, String msg, String fname) {
	mimeHeader(code, msg, mime_text_html, 0);
	htmlHeader("Error: " + msg);
	clientOutput.println("<body>\n" +
			   "<h1>" + code + " " + msg + "</h1>");
	if (fname != null)
	    clientOutput.println("Error when fetching URL: " + fname);
	
	if (mh != null) {
	    clientOutput.print("<p><pre>");
	    mh.print(clientOutput);
	}

	clientOutput.close();
    }

    final public void serviceRequest() {
	try {
	    mh = new MessageHeader(clientInput);
	    System.out.println(mh);
	    String cmd = mh.findValue(null);
	    if (cmd == null) {
		error(500, "Missing command " + mh, null);
		return;
	    }

	    int fsp = cmd.indexOf(' ');
	    int nsp = cmd.indexOf(' ', fsp + 1);

	    if (fsp < 0 || nsp < 0) {
		error(400, "Bad Request", cmd);
		return;
	    }

	    String get_or_post = cmd.substring(0, fsp);
	    String filename = cmd.substring(fsp + 1, nsp);

	    if (get_or_post.equalsIgnoreCase("get")) {
		getRequest(new URL(defaultContext, filename));
	    } else {
		error(500, "Unknown command: (" + cmd + ")", null);
		return;
	    }
	} catch(Exception e) {
	    e.printStackTrace();
	    error(500, "Exception: " + e, null);
	}
    }

    public static void main(String args[]) {
	try {
	    new HttpServer().startServer(80);
	} catch (Exception e) {
	    System.out.println("oops!\n" + e);
	}
    }
}
